from openpyxl.worksheet._writer import WorksheetWriter
from openpyxl.compat import safe_string
from operator import itemgetter
from .merge_cells import MergeCell, MergeCells


class SynoWorksheetWriter(WorksheetWriter):

    def __init__(self, ws, outputer):
        WorksheetWriter.__init__(self, ws)
        self.outputer = outputer

    def write_rows(self):
        xf = self.xf.send(True)

        all_rows = []
        row = []
        pre_row_index = 0

        def push_row(row_idx):
            nonlocal row
            row.sort(key=itemgetter(0))
            all_rows.append((row_idx, row))
            row = []

        def on_cell(row_index, col_index, json_str):
            nonlocal row, pre_row_index
            if row_index != pre_row_index:
                push_row(pre_row_index)

            row.append((col_index, json_str))
            pre_row_index = row_index

        self.outputer.parse_cells(on_cell)
        if row:
            push_row(pre_row_index)
        all_rows.sort(key=itemgetter(0), reverse=True)

        with xf.element('sheetData'):
            while all_rows:
                row_index, r = all_rows.pop()
                self.write_row(xf, r, row_index + 1)

        self.xf.send(None)  # return control to generator

    def write_row(self, xf, row, row_idx):
        ws = self.ws
        attrs = {'r': '%d' % row_idx}
        if row_idx in ws.row_dimensions:
            dim = ws.row_dimensions[row_idx]
            attrs.update(dict(dim))

        with xf.element('row', attrs):
            for cell in self.outputer.cells(row, row_idx):
                self._write_cell(xf, ws, cell, cell.has_style)

    def _write_cell(self, xf, worksheet, cell, styled):
        coordinate = cell.coordinate
        attributes = {'r': coordinate}
        if styled:
            attributes['s'] = '%d' % cell.xf_id

        if cell.data_type != 'f':
            attributes['t'] = cell.data_type

        value = cell._value

        if value == '' or value is None:
            with xf.element("c", attributes):
                return

        with xf.element('c', attributes):
            if cell.data_type == 'f':
                shared_formula = worksheet.formula_attributes.get(
                    coordinate, {})
                with xf.element('f', shared_formula):
                    if value is not None:
                        xf.write(value[1:])
                        value = None

            if cell.data_type == 's':
                value = worksheet.parent.shared_strings.add(value)
            with xf.element("v"):
                if value is not None:
                    xf.write(safe_string(value))

    def write_merged_cells(self):
        merged = self.outputer.get_merged_cells()
        if merged:
            cells = [MergeCell(ref) for ref in merged]
            self.xf.send(MergeCells(mergeCell=cells).to_tree())
