<?php
/**
 * This file is part of PHPPresentation - A pure PHP library for reading and writing
 * presentations documents.
 *
 * PHPPresentation is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPPresentation/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPPresentation
 * @copyright   2009-2015 PHPPresentation contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpPresentation\Style;

class SchemeColor extends Color
{
    const TRANSFORM_LUM = 'lum';
    const TRANSFORM_SHADE = 'shade';
    const TRANSFORM_TINT = 'tint';

    protected $value;
    protected $alpha = 100;
    protected $lumMod;
    protected $lumOff;
    protected $shade;
    protected $tint;
    protected $transform = '';

    private $supportTransform = array(
        self::TRANSFORM_LUM,
        self::TRANSFORM_SHADE,
        self::TRANSFORM_TINT
    );

    /**
     * @return string
     */
    public function getValue()
    {
        return $this->value;
    }

    /**
     * @param string $value
     */
    public function setValue($value)
    {
        $this->value = $value;
    }

    public function getAlpha()
    {
        return $this->alpha;
    }

    public function setAlpha($value)
    {
        $alpha = intval($value);
        if (0 > $alpha || 100 < $alpha) {
            $alpha = 100;
        }
        $this->alpha = $alpha;
    }

    public function getLumMod()
    {
        return $this->lumMod;
    }

    public function setLumMod($pValue)
    {
        if (is_int($pValue)) {
            $this->lumMod = intval($pValue);
        }
        return $this;
    }

    public function getLumOff()
    {
        return $this->lumOff;
    }

    public function setLumOff($pValue)
    {
        if (is_int($pValue)) {
            $this->lumOff = intval($pValue);
        }
        return $this;
    }

    public function getTint()
    {
        return $this->tint;
    }

    public function setTint($value)
    {
        $this->tint = $value;
        return $this;
    }

    public function setTransform($value)
    {
        if (in_array($value, $this->supportTransform)) {
            $this->transform = $value;
        }
        return $this;
    }

    public function getTransform()
    {
        return $this->transform;
    }
}
