<?php

namespace PhpOffice\PhpPresentation\Writer\PowerPoint2007;

use PhpOffice\Common\XMLWriter;
use PhpOffice\PhpPresentation\DocumentLayout;

class PptPresentation extends AbstractDecoratorWriter
{
    const MARGIN_LEFT_BASE_VALUE = 457200;
    const DEFAULT_FONT_SIZE = 1100;

    /**
     * @return \PhpOffice\Common\Adapter\Zip\ZipInterface
     */
    public function render()
    {
        // Create XML writer
        $objWriter = new XMLWriter(XMLWriter::STORAGE_MEMORY);

        // XML header
        $objWriter->startDocument('1.0', 'UTF-8', 'yes');

        // p:presentation
        $objWriter->startElement('p:presentation');
        $objWriter->writeAttribute('xmlns:a', 'http://schemas.openxmlformats.org/drawingml/2006/main');
        $objWriter->writeAttribute('xmlns:r', 'http://schemas.openxmlformats.org/officeDocument/2006/relationships');
        $objWriter->writeAttribute('xmlns:p', 'http://schemas.openxmlformats.org/presentationml/2006/main');

        // p:sldMasterIdLst
        $objWriter->startElement('p:sldMasterIdLst');

        // Add slide masters
        $relationId    = 1;
        $slideMasterId = 2147483648;

        $countMasterSlides = count($this->oPresentation->getAllMasterSlides());
        for ($inc = 1; $inc <= $countMasterSlides; $inc++) {
            // p:sldMasterId
            $objWriter->startElement('p:sldMasterId');
            $objWriter->writeAttribute('id', $slideMasterId);
            $objWriter->writeAttribute('r:id', 'rId' . $relationId++);
            $objWriter->endElement();

            // Increase identifier
            $slideMasterId += 12;
        }
        $objWriter->endElement();

        // theme
        $relationId++;

        $objWriter->startElement('p:notesMasterIdLst');
        $objWriter->startElement('p:notesMasterId');
        $objWriter->writeAttribute('r:id', 'rId' . $relationId++);
        $objWriter->endElement();
        $objWriter->endElement();

        // p:sldIdLst
        $objWriter->startElement('p:sldIdLst');
        // Write slides
        $slideCount = $this->oPresentation->getSlideCount();
        for ($i = 0; $i < $slideCount; ++$i) {
            // p:sldId
            $objWriter->startElement('p:sldId');
            $objWriter->writeAttribute('id', ($i + 256));
            $objWriter->writeAttribute('r:id', 'rId' . ($i + $relationId));
            $objWriter->endElement();
        }
        $objWriter->endElement();

        // p:sldSz
        $objWriter->startElement('p:sldSz');
        $objWriter->writeAttribute('cx', $this->oPresentation->getLayout()->getCX());
        $objWriter->writeAttribute('cy', $this->oPresentation->getLayout()->getCY());
        if ($this->oPresentation->getLayout()->getDocumentLayout() != DocumentLayout::LAYOUT_CUSTOM) {
            $objWriter->writeAttribute('type', $this->oPresentation->getLayout()->getDocumentLayout());
        }
        $objWriter->endElement();

        // p:notesSz
        $objWriter->startElement('p:notesSz');
        $objWriter->writeAttribute('cx', '6858000');
        $objWriter->writeAttribute('cy', '9144000');
        $objWriter->endElement();

        $objWriter->startElement('p:defaultTextStyle');

        $objWriter->startElement('a:defPPr');
        $objWriter->startElement('a:defRPr');
        $objWriter->writeAttribute('lang', $this->oPresentation->getLanguage());
        $objWriter->endElement();
        $objWriter->endElement();
        for ($i = 0; $i < 9; $i++) {
            $objWriter->startElement('a:lvl' . ($i + 1) . 'pPr');
            $objWriter->writeAttribute('marL', $i * self::MARGIN_LEFT_BASE_VALUE);
            $objWriter->writeAttribute('algn', 'l');
            //FIXME
            $objWriter->writeAttribute('defTabSz', '285750');
            $objWriter->writeAttribute('rtl', 0);
            $objWriter->writeAttribute('eaLnBrk', 1);
            $objWriter->writeAttribute('latinLnBrk', 0);
            $objWriter->writeAttribute('hangingPunct', 1);

            $objWriter->startElement('a:defRPr');
            $objWriter->writeAttribute('sz', self::DEFAULT_FONT_SIZE);
            $objWriter->writeAttribute('kern', 1200);
            $objWriter->startElement('a:solidFill');
            $objWriter->startElement('a:schemeClr');
            $objWriter->writeAttribute('val', 'tx1');
            $objWriter->endElement();
            $objWriter->endElement();
            $objWriter->startElement('a:latin');
            $objWriter->writeAttribute('typeface', '+mn-lt');
            $objWriter->endElement();
            $objWriter->startElement('a:ea');
            $objWriter->writeAttribute('typeface', '+mn-ea');
            $objWriter->endElement();
            $objWriter->startElement('a:cs');
            $objWriter->writeAttribute('typeface', '+mn-cs');
            $objWriter->endElement();
            $objWriter->endElement();
            $objWriter->endElement();
        }
        $objWriter->endElement();

        $objWriter->endElement();

        $this->oZip->addFromString('ppt/presentation.xml', $objWriter->getData());

        // Return
        return $this->oZip;
    }
}
