<?php

namespace PhpOffice\PhpPresentation\Writer\ProseMirror;

use PhpOffice\PhpPresentation\DocumentLayout;

class Doc extends Node
{
    public $type = 'doc';
    private $layouMap = array();
    private $masterMap = array();
    private $slideIds = array();
    private $comments = array();

    public function setData($phpPresentation)
    {
        $layout = $phpPresentation->getLayout();
        $this->setAttr('type', 'presentation');
        $this->setAttr('schema', 2);
        $this->setAttr('cx', $layout->getCX(DocumentLayout::UNIT_PIXEL));
        $this->setAttr('cy', $layout->getCY(DocumentLayout::UNIT_PIXEL));

        $masterContainer = new SlideContainer();
        $masterContainer->type = 'master_container';

        $phpMasters = $phpPresentation->getAllMasterSlides();
        foreach ($phpMasters as $index => $phpMaster) {
            $master = new Master($phpPresentation, $this);
            $master->setAttr('id', "$index");
            $master->setData($phpMaster);

            $this->masterMap[$index] = $master;

            $masterContainer->addContent($master);
        }
        $this->addContent($masterContainer);

        $slideContainer = new SlideContainer();
        $slideContainer->type = 'slide_container';
        $phpSlides = $phpPresentation->getAllSlides();

        $this->genAllSlidesId(count($phpSlides));

        $slideIdx = 0;
        foreach ($phpSlides as $phpSlide) {
            $slide = new Slide($phpPresentation);
            $phpLayout = $phpSlide->getSlideLayout();
            $layout = $this->layouMap[$phpLayout->getRelsIndex()];
            $slide->setAttr('layout_id', $layout->attrs->id);
            $slide->setAttr('master_id', $layout->attrs->master_id);
            $slide->setAttr('id', $this->slideIds[$slideIdx]);
            $slide->setAttr('is_visible', $phpSlide->isVisible());
            $slide->setIdx($slideIdx);
            $slide->setPresentation($this);
            $slide->setMaster($this->masterMap[$layout->attrs->master_id]);
            $slide->setLayout($layout);
            $slide->setData($phpSlide);

            $slideContainer->addContent($slide);
            $slideIdx++;

            foreach ($slide->getComments() as $comment) {
                $this->comments[] = $comment;
            }
        }
        $pageNumberInstanceCount = PageNumberSlideHolder::getInstanceCount();
        if (0 < $pageNumberInstanceCount) {
            $slideContainer->setAttr('page_num_enabled', true);
            $slideContainer->setAttr('page_num_skip_title', $slideIdx !== $pageNumberInstanceCount);
        }
        $this->addContent($slideContainer);
    }

    public function setLayoutMap($relsIndex, $layout)
    {
        $this->layouMap[$relsIndex] = $layout;
    }

    public function getSlideId($slideIdx)
    {
        if (0 > $slideIdx) {
            return $this->slideIds[0];
        }

        if (count($this->slideIds) < $slideIdx) {
            return $this->slideIds[count($this->slideIds) - 1];
        }

        return $this->slideIds[$slideIdx];
    }

    public function getLastSlideId()
    {
        return $this->slideIds[count($this->slideIds) - 1];
    }

    private function genAllSlidesId($slideCount)
    {
        for ($i = 0; $i < $slideCount; $i++) {
            array_push($this->slideIds, Utils::genSynoId());
        }
    }

    public function getComments()
    {
        return $this->comments;
    }
}

?>
