<?php

namespace PhpOffice\PhpPresentation\Writer\ProseMirror;

class Slide extends AbstractSlide
{
    public $type = 'slide';
    private $idx;
    private $presentation;
    private $master;
    private $layout;

    public function setData($phpSlide)
    {
        parent::setData($phpSlide);
        $this->setNote();
        $this->setTransition();
        $this->setAnimations();
    }

    public function setNote()
    {
        $note = new Note();
        $note->setSlide($this);
        $note->setData($this->phpSlide->getNote());
        $this->addContent($note);
    }

    protected function handleHolder($phpShape, $container)
    {
        $phpHolder = $phpShape->getPlaceHolder();
        switch ($phpHolder->getType()) {
            case 'title':
            case 'ctrTitle':
                $instance = new TitleSlideHolder($this, $container);
                break;
            case 'subTitle':
                $instance = new SubTitleSlideHolder($this, $container);
                break;
            case 'body':
                $instance = new BodySlideHolder($this, $container);
                break;
            case 'sldNum':
                $instance = new PageNumberSlideHolder($this, $container);
                break;
            default:
                //throw new \Exception("Not supported holder [" . $phpHolder->getType()  . "]");
                return;
        }

        $typeId = $this->layout->getTypeIdByHoderIdx($phpHolder->getIdx());
        $instance->setAttr('type_id', "$typeId");
        $instance->setData($phpShape);

        return $instance;
    }

    public function getBodyStyleAtLv($lv)
    {
        return $this->master->getBodyStyleAtLv($lv);
    }

    public function getTitleStyleAtLv($lv)
    {
        return $this->master->getTitleStyleAtLv($lv);
    }

    public function getSchemeColor($colorName)
    {
        return $this->master->getSchemeColor($colorName);
    }

    protected function createContentContainer()
    {
        $contentContainer = new ContentContainer();
        $contentContainer->type = 'slide_content_container';
        return $contentContainer;
    }

    public function getIdx()
    {
        return $this->idx;
    }

    public function setIdx($idx)
    {
        $this->idx = $idx;
    }

    public function getPresentation()
    {
        return $this->presentation;
    }

    public function setPresentation($presentation)
    {
        $this->presentation = $presentation;
    }

    public function setMaster($master)
    {
        $this->master = $master;
    }

    public function setLayout($layout)
    {
        $this->layout = $layout;
    }

    public function getLayout()
    {
        return $this->layout;
    }

    public function getSchemeFont($isMajor, $lang = '')
    {
        return $this->master->getSchemeFont($isMajor);
    }
}

?>
