<?php

namespace PhpOffice\PhpPresentation\Writer\ProseMirror;

use PhpOffice\PhpPresentation\Style\Bullet;
use PhpOffice\PhpPresentation\Style\Fill;

class TableBox extends FloatingObject
{
    public $type = 'table_box';

    private function getIndent($indent, $levelIndentMapping)
    {
        foreach ($levelIndentMapping as $level => $levelIndent) {
            if ($indent <= $levelIndent) {
                return $level * 30 . 'px';
            }
        }
        return '';
    }

    protected function setContent()
    {
        $tableBlock = new Node(); 
        $tableBlock->type = 'table_block';

        $table = new Node();
        $table->type = 'table';

        $table->setAttr('height', $this->attrs->transform->height . 'px');
        $table->setAttr('width', $this->attrs->transform->width . 'px');
        $this->setTableContent($table);

        $tableBlock->addContent($table);
        $this->addContent($tableBlock);
    }

    private function setTableContent($table)
    {
        $colGroup = array();
        $grid = array();

        $styleId = $this->phpShape->getStyleId();
        if (isset($styleId)) {
            $tableStyle = $this->slide->getTableStyle($styleId);
            $tableProperties = $this->phpShape->getProperties();
        }

        $phpRows = $this->phpShape->getRows();
        $colCount = $this->phpShape->getNumColumns();
        $rowCount = count($phpRows);

        for ($i = 0; $i < $rowCount; $i++) {
            $phpRow = $phpRows[$i];
            $gridRow = array();

            for ($j = 0; $j < $colCount; $j++) {
                if (0 === $i) {
                    $colGroupElm = new \stdClass();
                    $colGroupElm->width = $phpRow->getCell($j)->getWidth() . 'px';
                    $colGroup[] = $colGroupElm;
                }

                $gridCell = new \stdClass();
                $gridCell->colspan = 1;
                $gridCell->dx = 0;
                $gridCell->dy = 0;
                $gridCell->real = true;
                $gridCell->rowspan = 1;
                $gridCell->x = 0;
                $gridCell->y = 0;

                $gridRow[] = $gridCell;
            }
            $grid[] = $gridRow;
        }

        foreach ($phpRows as $rowIdx => $phpRow) {
            $row = new Node();
            $row->type = 'table_row';
            $row->setAttr('height', $phpRow->getHeight() . 'px');

            $phpCells = $phpRow->getCells();
            foreach ($phpCells as $colIdx => $phpCell) {
                $colSpan = intval($phpCell->getColSpan());
                $rowSpan = intval($phpCell->getRowSpan());

                if ($grid[$rowIdx][$colIdx]->real) {
                    $grid[$rowIdx][$colIdx]->y = $rowIdx;

                    $x = 0;
                    for ($k = $colIdx - 1; $k >= 0; $k--) {
                        if ($grid[$rowIdx][$k]->real) {
                            $x = $grid[$rowIdx][$k]->x + 1;
                            break;
                        }
                    }
                    $grid[$rowIdx][$colIdx]->x = $x;
                }

                for ($i = 0; $i < $colSpan; $i++) {
                    $grid[$rowIdx][$colIdx + $i]->colspan = $colSpan;
                    if (0 === $i) {
                        continue;
                    }
                    $grid[$rowIdx][$colIdx + $i]->dx = $i;
                    $grid[$rowIdx][$colIdx + $i]->real = false;
                    $grid[$rowIdx][$colIdx + $i]->x = $grid[$rowIdx][$colIdx]->x;
                    $grid[$rowIdx][$colIdx + $i]->y = $grid[$rowIdx][$colIdx]->y;
                }

                for ($i = 0; $i < $rowSpan; $i++) {
                    $grid[$rowIdx + $i][$colIdx]->rowspan = $rowSpan;
                    if (0 === $i) {
                        continue;
                    }
                    $grid[$rowIdx + $i][$colIdx]->dy = $i;
                    $grid[$rowIdx + $i][$colIdx]->real = false;
                    $grid[$rowIdx + $i][$colIdx]->x = $grid[$rowIdx][$colIdx]->x;
                    $grid[$rowIdx + $i][$colIdx]->y = $grid[$rowIdx][$colIdx]->y;
                }

                if (!$grid[$rowIdx][$colIdx]->real) {
                    continue;
                }

                $cell = new Node();
                $cell->type = 'table_cell';

                $styles = $this->setCellAttrs($phpCell, $cell, $rowCount, $colCount, $rowIdx, $colIdx, $tableProperties, $tableStyle);
                $phpParagraphs = $phpCell->getParagraphs();
                //
                $preLevel = -1;
                $container = $cell;
                $listStack = array();
                $levelIndentMapping = array();
                foreach ($phpParagraphs as $phpParagraph) {
                    $paragraph = new Paragraph();
                    $paragraph->setSlide($this->slide);
                    $paragraph->setCellStyles($styles);
                    $paragraph->setData($phpParagraph);

                    $level = $phpParagraph->getAlignment()->getLevel() + 1;

                    $bulletType = $phpParagraph->getBulletStyle()->getBulletType();
                    if (Bullet::TYPE_NOTSET === $bulletType) {
                        $bulletType = Bullet::TYPE_NONE;
                    }

                    if (Bullet::TYPE_NONE !== $bulletType) {
                        if (!isset($levelIndentMapping[$level])) {
                            $levelIndentMapping[$level] = $phpParagraph->getAlignment()->getMarginLeft();
                        }

                        if ($preLevel < $level) {
                            if (-1 !== $preLevel) {
                                array_push($listStack, $list);
                            }
                            $list = ('numeric' === $bulletType) ? new OrderedList() : new BulletList();
                            $container->addContent($list);
                        } else if ($preLevel > $level) {
                            $list = array_pop($listStack);
                        }
                        $preLevel = $level;
                        $listItem = new ListItem();
                        $list->addContent($listItem);
                        $container = $listItem;
                    } else {
                        if (0 < $phpParagraph->getAlignment()->getIndent()) {
                            $paragraph->setAttr('paddingLeft', $this->getIndent($phpParagraph->getAlignment()->getIndent(), $levelIndentMapping));
                        }
                        $preLevel = -1;
                        $container = $cell;
                    }

                    $container->addContent($paragraph);
                }

                if (0 === count($phpParagraphs)) {
                    $paragraph = new Paragraph();
                    $paragraph->setSlide($this->slide);
                    $container->addContent($paragraph);
                }
                //
                $row->addContent($cell);
            }
            $table->addContent($row);
        }

        $table->setAttr('colgroup', $colGroup);
        $table->setAttr('grid', $grid);
        $table->setAttr('synotable', Utils::genSynoId());
    }

    private function setCellAttrs($phpCell, $cell, $rowCount, $colCount, $rowIdx, $colIdx, $tableProperties, $tableStyle)
    {
        $styles = array();
        $borderStyles = array();
        $colSpan = intval($phpCell->getColSpan());
        $rowSpan = intval($phpCell->getRowSpan());

        $vAlign = 'middle';
        if (0 < count($phpCell->getParagraphs())) {
            switch ($phpCell->getParagraph(0)->getAlignment()->getVertical()) {
                case 't':
                    $vAlign = 'top';
                    break;
                case 'b':
                    $vAlign = 'bottom';
                    break;
                case 'ctr':
                default:
                    break;
            }
        }

        $cell->setAttr('colspan', (0 < $colSpan) ? $colSpan : 1);
        $cell->setAttr('rowspan', (0 < $rowSpan) ? $rowSpan : 1);
        $cell->setAttr('verticalAlign', $vAlign);

        if (isset($tableStyle->tblBg)) {
            $oColor = $tableStyle->tblBg->fillColor;
        }
        if (isset($tableStyle->wholeTbl)) {
            $style = $tableStyle->wholeTbl;
            if (isset($style->fillColor)) {
                $oColor = $style->fillColor;
            }
            $this->slide->getTextStyles($style->font, $styles);
            $this->slide->getBorderStyles($style->borders, $borderStyles);
        }
        if ($tableProperties['bandRow'] && (!$tableProperties['firstRow'] || 0 !== $rowIdx)) {
            if (0 === $rowIdx % 2) {
                $style = $tableStyle->band2H;
            } else {
                $style = $tableStyle->band1H;
            }
            if (isset($style)) {
                if (isset($style->fillColor)) {
                    $oColor = $style->fillColor;
                }
                $this->slide->getTextStyles($style->font, $styles);
                $this->slide->getBorderStyles($style->borders, $borderStyles);
            }
        }
        if ($tableProperties['firstRow'] && 0 === $rowIdx && isset($tableStyle->firstRow)) {
            $style = $tableStyle->firstRow;
            if (isset($style->fillColor)) {
                $oColor = $style->fillColor;
            }
            $this->slide->getTextStyles($style->font, $styles);
            $this->slide->getBorderStyles($style->borders, $borderStyles);
        }
        if ($tableProperties['lastRow'] && $rowCount - 1 === $rowIdx && isset($tableStyle->lastRow)) {
            $style = $tableStyle->lastRow;
            if (isset($style->fillColor)) {
                $oColor = $style->fillColor;
            }
            $this->slide->getTextStyles($style->font, $styles);
            $this->slide->getBorderStyles($style->borders, $borderStyles);
        }
        if ($tableProperties['firstCol'] && 0 == $colIdx) {
            $style = $tableStyle->firstCol;
            if (isset($style)) {
                $this->slide->getTextStyles($style->font, $styles);
                $this->slide->getBorderStyles($style->borders, $borderStyles);
                $oColor = $style->fillColor;
            }
        }

        $cellFill = $phpCell->getFill();
        if (Fill::FILL_NONE !== $cellFill->getFillType()) {
            $oColor = $cellFill->getStartColor();
        }
        $this->slide->getBorderStyles($phpCell->getBorders(), $borderStyles);

        if (isset($oColor)) {
            $cell->setAttr('backgroundColor', '#' . $this->slide->getColor($oColor));
        }

        $cell->setAttr('borderTopColor', '#7b8187');
        $cell->setAttr('borderTopStyle', 'solid');
        $cell->setAttr('borderTopWidth', '1px');

        $cell->setAttr('borderRightColor', '#7b8187');
        $cell->setAttr('borderRightStyle', 'solid');
        $cell->setAttr('borderRightWidth', '1px');

        $cell->setAttr('borderBottomColor', '#7b8187');
        $cell->setAttr('borderBottomStyle', 'solid');
        $cell->setAttr('borderBottomWidth', '1px');

        $cell->setAttr('borderLeftColor', '#7b8187');
        $cell->setAttr('borderLeftStyle', 'solid');
        $cell->setAttr('borderLeftWidth', '1px');

        foreach ($borderStyles as $key => $value) {
            $cell->setAttr($key, $value);
        }

        return $styles;
    }
}

?>
