<?php

namespace PhpOffice\PhpPresentation\Writer\ProseMirror;

use PhpOffice\PhpPresentation\Style\Font;

class Utils
{
    public static function genSynoId($idLen = 10)
    {
        $possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
        $length = strlen($possible);
        $id = '';

        for ($i = 0; $i < $idLen; $i++) {
            $id .= substr($possible, rand(0, $length - 1), 1);
        }

        return $id;
    }

    public static function getTextStyles($font, &$styles, $getTypeface = false, $slide = null)
    {
        if (!($font instanceof Font)) {
            return;
        }

        if ($getTypeface) {
            $langFontMap = $font->getLangFontMap();
            $typeface = $textLangFontMap['a:latin'];
            if (!isset($typeface) || '' === $typeface) {
                foreach ($langFontMap as $lang => $typeface) {
                    if (isset($typeface) && '' != $typeface) {
                        break;
                    }
                }
            }
            if (isset($typeface) && '' != $typeface) {
                $styles['font_family'] = $typeface;
            }
        }

        if (Font::STATE_NOTSET !== $font->getBold()) {
            $styles['strong'] = (Font::STATE_ON === $font->getBold());
        }
        if (Font::STATE_NOTSET !== $font->getItalic()) {
            $styles['em'] = (Font::STATE_ON === $font->getItalic());
        }

        if (!is_null($slide)) {
            if ('' !== $font->getIdx()) {
                $styles['font_family'] = $slide->getSchemeFont('major' === $font->getIdx());
            }

            $color = $slide->getColor($font->getColor());
            if (isset($color)) {
                $styles['color'] = "#$color";
            }

            $highlight = $slide->getColor($font->getHighlight());
            if (isset($highlight)) {
                $styles['background_color'] = "#$highlight";
            }
        }

        if (Font::UNDERLINE_NOTSET !== $font->getUnderline()) {
            $styles['underline'] = (Font::UNDERLINE_NONE !== $font->getUnderline());
        }

        if (0 !== $font->getSize()) {
            $styles['font_size'] = $font->getSize() . "pt";
        }

        if (Font::STATE_NOTSET !== $font->getStrikethrough()) {
            $styles['strike'] = (Font::STATE_ON === $font->getStrikethrough());
        }

        if (0 != $font->getBaseline()) {
            $mark = new \stdClass();
            if (0 > $font->getBaseline()) {
                $styles['subscript'] = true;
            } else {
                $styles['superscript'] = true;
            }
        }
    }

    public static function styleToMarks(&$styles, &$marks)
    {
        if (!is_array($styles)) {
            return;
        }

        foreach ($styles as $name => $value) {
            $mark = new \stdClass();
            $mark->_ = $name;
            if (is_bool($value) && !$value) {
                continue;
            } else {
                $mark->value = $value;
            }
            array_push($marks, $mark);
        }
    }

    public static function getIndent($indent, $levelIndentMapping)
    {
        if (!is_array($levelIndentMapping) && !is_object($levelIndentMapping)) {
            return '';
        }
        foreach ($levelIndentMapping as $level => $levelIndent) {
            if ($indent <= $levelIndent) {
                return $level * 30 . 'px';
            }
        }
        return '';
    }
}

?>
