<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Writer\HTML_ProseMirror\Element;

use PhpOffice\PhpWord\Writer\HTML_ProseMirror\Element\Container;

/**
 * Header element HTML writer
 *
 * @since 0.10.0
 */
class Header extends AbstractElement
{
    protected $tag = 'header';

    /**
     * Write header
     *
     * @return string
     */
    public function write()
    {
        if (!$this->check()) {
            return '';
        }
        $content = "<{$this->tag} type=\"{$this->getType()}\" style=\"width: 622px;\">" . PHP_EOL;
        $writer = new Container($this->parentWriter, $this->element);
        $content .= $writer->write();
        $content .= "</{$this->tag}>" . PHP_EOL;

        return $content;
    }

    protected function check() {
        if (!$this->element instanceof \PhpOffice\PhpWord\Element\Header) {
            return false;
        }
        return true;
    }

    private function getType() {
        switch ($this->element->getType())
        {
            case 'default':
                return 'normal';
            case 'first':
                return 'cover';
            case 'even':
                return 'even';
            default:
                return 'normal';
        }
    }
}
