<?php
namespace PhpOffice\PhpWord\Writer\HTML_ProseMirror\Part;

use PhpOffice\PhpWord\Writer\HTML_ProseMirror\Element\Container;
use PhpOffice\PhpWord\Writer\HTML_ProseMirror\Element\Header;
use PhpOffice\PhpWord\Writer\HTML_ProseMirror\Element\Footer;

class Document extends AbstractPart {
    public function write() {
        $phpWord = $this->getParentWriter()->getPhpWord();
        $synodeco = $phpWord->synoDecorator;
        $content = '';
        $sections = $phpWord->getSections();
        $style;
        $pageSetup = '';
        $first_section;

        //Write syno_invisible part
        $content = "<div syno_invisible=\"true\" style=\"display: none;\"></div>" . PHP_EOL;

        //Write synoedit part
        $content .= '<div synoedit="true">' . PHP_EOL;
        foreach ($sections as $section) {
            if ($first_section === null) {
                $first_section = $section;
            }
            $writer = new Container($this->getParentWriter(), $section);
            $content .= $writer->write();

        }
        $content .= '</div>' . PHP_EOL;

        if ($first_section !== null) {
            //Write synodecorate part
            $this->fillHeaderFooters($first_section);
            $headers = $this->reOrder($first_section->getHeaders());
            $footers = $this->reOrder($first_section->getFooters());
            $placeshow = $synodeco['place-show'] ? 'true' : 'false';
            $placecover = $synodeco['place-cover'] ? 'true' : 'false';
            // FIXME place-even
            $numshow = $synodeco['num-show'] ? 'true' : 'false';
            $numcover = ($synodeco['num-cover'] && $synodeco['place-cover']) ? 'true' : 'false';
            $numpos = $synodeco['num-pos'];

            $content .= "<div synodecorate=\"true\" place-show=\"{$placeshow}\" place-cover=\"{$placecover}\" place-even=\"false\" num-show=\"{$numshow}\" num-cover=\"{$numcover}\" num-even=\"false\" num-pos=\"{$numpos}\">" . PHP_EOL;
            if (count($headers) > 0 || count($footers) > 0) {
                foreach ($headers as $header) {
                    $writer = new Header($this->getParentWriter(), $header);
                    $content .= $writer->write();
                }
                foreach ($footers as $footer) {
                    $writer = new Footer($this->getParentWriter(), $footer);
                    $content .= $writer->write();
                }
            } else {
                $content .= $this->getDefaultDecorateContent();
            }
            $content .= "<div pagenum=\"true\" type=\"normal\"><span synonumtext=\"true\"></span></div>" . PHP_EOL;
            $content .= "</div>" . PHP_EOL;

            //Write page setup part
            $style = $first_section->getStyle();
            $pageSetup = '{"top":' . $this->twip2cm($style->getMarginTop()) .
',"bottom":' . $this->twip2cm($style->getMarginBottom()) .
',"left":' . $this->twip2cm($style->getMarginLeft()) .
',"right":' . $this->twip2cm($style->getMarginRight()) .
',"orientation":"' . $style->getOrientation() . '"' .
',"page_type":"' . $this->getPaperSize($style->getPageSizeW(), $style->getPageSizeH(), $style->getOrientation()) . '"' .
'}';
$content = '<div pageSetup="' . $pageSetup . '">' . PHP_EOL . $content . '</div>';
        }
        return $content;
    }

    private function getDefaultDecorateContent() {
        $content =  "<header type=\"cover\" style=\"width: 622px;\">" .PHP_EOL .
            "<p textblock=\"true\"></p>" .PHP_EOL .
            "</header>" .PHP_EOL .
            "<header type=\"even\" style=\"width: 622px;\">" .PHP_EOL .
            "<p textblock=\"true\"></p>" .PHP_EOL .
            "</header>" .PHP_EOL .
            "<header type=\"normal\" style=\"width: 622px;\">" .PHP_EOL .
            "<p textblock=\"true\"></p>" .PHP_EOL .
            "</header>" .PHP_EOL .
            "<footer type=\"cover\" style=\"width: 622px;\">" .PHP_EOL .
            "<p textblock=\"true\"></p>" .PHP_EOL .
            "</footer>" .PHP_EOL .
            "<footer type=\"even\" style=\"width: 622px;\">" .PHP_EOL .
            "<p textblock=\"true\"></p>" .PHP_EOL .
            "</footer>" .PHP_EOL .
            "<footer type=\"normal\" style=\"width: 622px;\">" .PHP_EOL .
            "<p textblock=\"true\"></p>" .PHP_EOL .
            "</footer>" .PHP_EOL;
        return $content;
    }

    private function checkDifferentCover($cover) {
        if ($cover === null) {
            return false;
        }

        $elements = $cover->getElements();
        if (count($elements) === 0 || (count($elements) === 1 && get_Class($elements[0]) === 'PhpOffice\\PhpWord\\Element\\TextBreak')) {
            return false;
        }
        return true;
    }

    private function getTypeArray($headerfooters) {
        $typeArray = array("even"=> false, "default"=>false, "first"=>false);
        foreach ($headerfooters as $item) {
            $type = $item->getType();
            if ('even' === $type) {
                $typeArray["even"] = true;
            } else if ('default' === $type) {
                $typeArray["default"] = true;
            } else if ("first" == $type) {
                $typeArray["first"] = true;
            }
        }
        return $typeArray;
    }

    private function fillHeaderFooters($section) {
        $headers = $this->getTypeArray($section->getHeaders());
        foreach($headers as $key => $value) {
            if ($value === false) {
                $section->addHeader($key);
            }
        }
        $footers = $this->getTypeArray($section->getFooters());
        foreach($footers as $key => $value) {
            if ($value === false) {
                $section->addFooter($key);
            }
        }
    }

    private function reOrder($headerfooters) {
        if (count($headerfooters) <= 0) {
            return $headerfooters;
        }
        $cover;
        $even;
        $normal;
        foreach ($headerfooters as $item) {
            $type = $item->getType();
            if ('default' === $type) {
                $normal = $item;
            } else if ('first' === $type) {
                $cover = $item;
            } else if ('even' === $type) {
                $even = $item;
            }
        }
        return array($cover, $even, $normal);
    }

    private function twip2cm($twip) {
        return round($twip * 0.0017638888888889, 2);
    }

    private function getPaperSize($width, $height, $orientation) {
        $w = ($orientation == 'landscape') ? $height : $width;
        $h = ($orientation == 'landscape') ? $width : $height;

        $sizes = array(
            "letter" => array("w"=>12240, "h"=>15840),
            "tabloid" => array("w"=>15840, "h"=>24480),
            "legal" => array("w"=>12240, "h"=>20160),
            "statement" => array("w"=>7920, "h"=>12240),
            "executive" => array("w"=>10440, "h"=>15120),
            "folio" => array("w"=>12240, "h"=>18720),
            "a3" => array("w"=>16838, "h"=>23811),
            "a4" => array("w"=>11906, "h"=>16838),
            "a5" => array("w"=>8391, "h"=>11906),
            "b4" => array("w"=>14173, "h"=>20013),
            "b5" => array("w"=>9978, "h"=>14173)
        );
        $minSize;
        $minDiff = INF;
        foreach ($sizes as $size => $value) {
            $diff = ($w - $value['w'])**2 + ($h - $value['h'])**2;
            if ($diff < $minDiff) {
                $minSize = $size;
                $minDiff = $diff;
            }
        }
        return $minSize;
    }
}
