<?php
/**
 * This file is part of PHPWord - A pure PHP library for reading and writing
 * word processing documents.
 *
 * PHPWord is free software distributed under the terms of the GNU Lesser
 * General Public License version 3 as published by the Free Software Foundation.
 *
 * For the full copyright and license information, please read the LICENSE
 * file that was distributed with this source code. For the full list of
 * contributors, visit https://github.com/PHPOffice/PHPWord/contributors.
 *
 * @link        https://github.com/PHPOffice/PHPWord
 * @copyright   2010-2014 PHPWord contributors
 * @license     http://www.gnu.org/licenses/lgpl.txt LGPL version 3
 */

namespace PhpOffice\PhpWord\Writer\Word2007\Element;

/**
 * Text element writer
 *
 * @since 0.10.0
 */
class Text extends Inline
{
    /**
     * Write text element.
     *
     * @return void
     */
    public function write()
    {
        $xmlWriter = $this->getXmlWriter();
        $element = $this->getElement();
        if (!$element instanceof \PhpOffice\PhpWord\Element\Text) {
            return;
        }

        $this->startElementP();

        $this->writeCommentRangeStart($xmlWriter, $element);
        $this->writeRevisionOpenTag($xmlWriter, $element);

        $xmlWriter->startElement('w:r');
        if ($element->isHidden()) {
            $xmlWriter->writeAttribute('syno-hidden', true);
        }

        $this->writeFontStyle();

        $revision = $element->getRevision();
        $openTag = 'w:t';
        if (is_array($revision) && ($revision['op'] === 'del')) {
            $openTag = 'w:delText';
        }
        $xmlWriter->startElement($openTag);
        $xmlWriter->writeAttribute('xml:space', 'preserve');
        $xmlWriter->writeRaw($this->getText($element->getText()));
        $xmlWriter->endElement(); // w:t or w:delText
        $xmlWriter->endElement(); // w:r

        $this->writeRevisionCloseTag($xmlWriter, $element);
        $this->writeCommentRangeEnd($xmlWriter, $element);

        $this->endElementP(); // w:p
    }
}
